@@ -11,29 +11,16 @@ package Net::BitTorrent::Protocol::BEP03::Packets;
     use vars qw[@EXPORT_OK %EXPORT_TAGS];
     use Exporter qw[];
     *import = *import = *Exporter::import;
-    @EXPORT_OK = qw[build_handshake build_keepalive build_choke build_unchoke
-        build_interested  build_not_interested build_have build_bitfield
-        build_request build_piece build_cancel build_port build_suggest
-        build_allowed_fast build_reject build_have_all build_have_none
-        build_extended parse_packet _parse_handshake _parse_keepalive
-        _parse_choke _parse_unchoke _parse_interested _parse_not_interested
-        _parse_have _parse_bitfield _parse_request _parse_piece _parse_cancel
-        _parse_port _parse_suggest _parse_have_all _parse_have_none
-        _parse_reject _parse_allowed_fast _parse_extended HANDSHAKE KEEPALIVE
-        CHOKE UNCHOKE INTERESTED NOT_INTERESTED HAVE BITFIELD REQUEST PIECE
-        CANCEL PORT SUGGEST HAVE_ALL HAVE_NONE REJECT ALLOWED_FAST EXTPROTOCOL
-    ];
     %EXPORT_TAGS = (
-        all   => [@EXPORT_OK],
         build => [
-            qw[build_handshake build_keepalive build_choke build_unchoke
-                build_interested  build_not_interested build_have
+            qw[ build_handshake build_keepalive build_choke build_unchoke
+                build_interested build_not_interested build_have
                 build_bitfield build_request build_piece build_cancel
                 build_port build_suggest build_allowed_fast build_reject
                 build_have_all build_have_none build_extended]
         ],
         parse => [
-            qw[parse_packet _parse_handshake _parse_keepalive
+            qw[ parse_packet _parse_handshake _parse_keepalive
                 _parse_choke _parse_unchoke _parse_interested
                 _parse_not_interested _parse_have _parse_bitfield
                 _parse_request _parse_piece _parse_cancel _parse_port
@@ -41,32 +28,36 @@ package Net::BitTorrent::Protocol::BEP03::Packets;
                 _parse_reject _parse_allowed_fast _parse_extended]
         ],
         types => [
-            qw[HANDSHAKE KEEPALIVE CHOKE UNCHOKE INTERESTED NOT_INTERESTED
-                HAVE BITFIELD REQUEST PIECE CANCEL PORT SUGGEST HAVE_ALL
-                HAVE_NONE REJECT ALLOWED_FAST EXTPROTOCOL]
-            ]
-
-            # XXX - Move MSE-related functions from N::B::Peer
-            #mse => [qw[  ]]
+            qw[ $HANDSHAKE $KEEPALIVE $CHOKE $UNCHOKE $INTERESTED
+                $NOT_INTERESTED $HAVE $BITFIELD $REQUEST $PIECE $CANCEL $PORT
+                $SUGGEST $HAVE_ALL $HAVE_NONE $REJECT $ALLOWED_FAST
+                $EXTPROTOCOL]
+        ]
     );
-    sub HANDSHAKE      {-1}
-    sub KEEPALIVE      {''}
-    sub CHOKE          {0}
-    sub UNCHOKE        {1}
-    sub INTERESTED     {2}
-    sub NOT_INTERESTED {3}
-    sub HAVE           {4}
-    sub BITFIELD       {5}
-    sub REQUEST        {6}
-    sub PIECE          {7}
-    sub CANCEL         {8}
-    sub PORT           {9}
-    sub SUGGEST        {13}
-    sub HAVE_ALL       {14}
-    sub HAVE_NONE      {15}
-    sub REJECT         {16}
-    sub ALLOWED_FAST   {17}
-    sub EXTPROTOCOL    {20}
+    @EXPORT_OK = sort map { @$_ = sort @$_; @$_ } values %EXPORT_TAGS;
+    $EXPORT_TAGS{'all'} = \@EXPORT_OK;
+
+    #
+    our $HANDSHAKE      = -1;
+    our $KEEPALIVE      = '';
+    our $CHOKE          = 0;
+    our $UNCHOKE        = 1;
+    our $INTERESTED     = 2;
+    our $NOT_INTERESTED = 3;
+    our $HAVE           = 4;
+    our $BITFIELD       = 5;
+    our $REQUEST        = 6;
+    our $PIECE          = 7;
+    our $CANCEL         = 8;
+    our $PORT           = 9;
+    our $SUGGEST        = 13;
+    our $HAVE_ALL       = 14;
+    our $HAVE_NONE      = 15;
+    our $REJECT         = 16;
+    our $ALLOWED_FAST   = 17;
+    our $EXTPROTOCOL    = 20;
+
+    #
     my $info_hash_constraint;
 
     sub build_handshake ($$$) {
@@ -97,6 +88,7 @@ package Net::BitTorrent::Protocol::BEP03::Packets;
 
     sub build_bitfield ($) {
         my ($bitfield) = @_;
+        $bitfield = pack 'b*', $bitfield->to_Bin if blessed $bitfield;
         if ((!$bitfield) || (unpack('b*', $bitfield) !~ m[^[01]+$])) {
             carp sprintf 'Malformed bitfield passed to %s::build_bitfield()',
                 __PACKAGE__;
@@ -127,8 +119,8 @@ package Net::BitTorrent::Protocol::BEP03::Packets;
         return pack('Nca*', length($packed) + 1, 6, $packed);
     }
 
-    sub build_piece ($$$) {
-        my ($index, $offset, $data) = @_;
+    sub build_piece ($$$$) {
+        my ($index, $offset, $length, $data) = @_;
         if ((!defined $index) || ($index !~ m[^\d+$])) {
             carp sprintf '%s::build_piece() requires an index parameter',
                 __PACKAGE__;
@@ -139,11 +131,22 @@ package Net::BitTorrent::Protocol::BEP03::Packets;
                 __PACKAGE__;
             return;
         }
+        if ((!defined $length) || ($length !~ m[^\d+$])) {
+            carp sprintf '%s::build_piece() requires an length parameter',
+                __PACKAGE__;
+            return;
+        }
         if (!$data or !$$data) {
             carp sprintf '%s::build_piece() requires data to work with',
                 __PACKAGE__;
             return;
         }
+        if ($length != length $$data) {
+            carp sprintf
+                'Incorrect data length or incomplete data block passed to %s::build_piece( %d, %d, %d, {%d bytes} )',
+                __PACKAGE__, $index, $offset, $length, length $$data;
+            return;
+        }
         my $packed = pack('N2a*', $index, $offset, $$data);
         return pack('Nca*', length($packed) + 1, 7, $packed);
     }
@@ -240,23 +243,23 @@ package Net::BitTorrent::Protocol::BEP03::Packets;
         my $packet = pack('ca*', $msgID, bencode($data));
         return pack('Nca*', length($packet) + 1, 20, $packet);
     }
-    my %parse_packet_dispatch = (&KEEPALIVE      => \&_parse_keepalive,
-                                 &CHOKE          => \&_parse_choke,
-                                 &UNCHOKE        => \&_parse_unchoke,
-                                 &INTERESTED     => \&_parse_interested,
-                                 &NOT_INTERESTED => \&_parse_not_interested,
-                                 &HAVE           => \&_parse_have,
-                                 &BITFIELD       => \&_parse_bitfield,
-                                 &REQUEST        => \&_parse_request,
-                                 &PIECE          => \&_parse_piece,
-                                 &CANCEL         => \&_parse_cancel,
-                                 &PORT           => \&_parse_port,
-                                 &SUGGEST        => \&_parse_suggest,
-                                 &HAVE_ALL       => \&_parse_have_all,
-                                 &HAVE_NONE      => \&_parse_have_none,
-                                 &REJECT         => \&_parse_reject,
-                                 &ALLOWED_FAST   => \&_parse_allowed_fast,
-                                 &EXTPROTOCOL    => \&_parse_extended
+    my %parse_packet_dispatch = ($KEEPALIVE      => \&_parse_keepalive,
+                                 $CHOKE          => \&_parse_choke,
+                                 $UNCHOKE        => \&_parse_unchoke,
+                                 $INTERESTED     => \&_parse_interested,
+                                 $NOT_INTERESTED => \&_parse_not_interested,
+                                 $HAVE           => \&_parse_have,
+                                 $BITFIELD       => \&_parse_bitfield,
+                                 $REQUEST        => \&_parse_request,
+                                 $PIECE          => \&_parse_piece,
+                                 $CANCEL         => \&_parse_cancel,
+                                 $PORT           => \&_parse_port,
+                                 $SUGGEST        => \&_parse_suggest,
+                                 $HAVE_ALL       => \&_parse_have_all,
+                                 $HAVE_NONE      => \&_parse_have_none,
+                                 $REJECT         => \&_parse_reject,
+                                 $ALLOWED_FAST   => \&_parse_allowed_fast,
+                                 $EXTPROTOCOL    => \&_parse_extended
     );
 
     sub parse_packet ($) {
@@ -269,7 +272,7 @@ package Net::BitTorrent::Protocol::BEP03::Packets;
         my ($packet);
         if (unpack('c', $$data) == 0x13) {
             my @payload = _parse_handshake(substr($$data, 0, 68, ''));
-            $packet = {type           => HANDSHAKE,
+            $packet = {type           => $HANDSHAKE,
                        packet_length  => 68,
                        payload_length => 48,
                        payload        => @payload
@@ -817,6 +820,6 @@ L<clarification of the CCA-SA3.0|http://creativecommons.org/licenses/by-sa/3.0/u
 Neither this module nor the L<Author|/Author> is affiliated with BitTorrent,
 Inc.
 
-=for rcs $Id: Packets.pm a7f61f8 2010-06-27 02:13:37Z sanko@cpan.org $
+=for rcs $Id: Packets.pm f1e1d47 2010-07-24 02:23:43Z sanko@cpan.org $
 
 =cut
